package be.AM4DP;

import be.ASPL.presentationlayer.ASPLListFactory;
import be.ASPL.presentationlayer.IEEEDoubleValue;
import be.ASPL.presentationlayer.InitialPresentation;
import be.ASPL.presentationlayer.OnLinePresentationFunctionCharacteristics;
import be.ASPL.presentationlayer.LogicalValue;
import be.ASPL.presentationlayer.ASPLException;
import be.SIRAPRISE.client.ConnectionClosedException;
import be.SIRAPRISE.client.DBConnection;
import be.SIRAPRISE.client.TransactionMode;
import be.SIRAPRISE.messages.ErrorMessageException;
import be.SIRAPRISE.util.DDLCapture;
import be.SIRAPRISE.util.SnapShotIsolationMode;

/**
 * The function to produce an overview of defined salary grades
 * 
 * @author Erwin
 */
@InitialPresentation(initialPresentationClass = SalaryGradesOverviewPresentation.class)
@OnLinePresentationFunctionCharacteristics(leaveFunctionOnExit = false, descriptionLocalizationKey = "SalaryGradesOverview.Text")
public class SalaryGradesOverview extends SalaryGradesWebClientFunction {

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doInitialLogic()
	 */
	public void doInitialLogic ( ) throws ASPLException {
		doPresentationLogic();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doPresentationLogic()
	 */
	public void doPresentationLogic ( ) throws ASPLException {
		
		try (DBConnection dbc = getDBConnection();) {
			String query = RELVARNAMES.GRD;

			IEEEDoubleValue wv_lowerLimit = (IEEEDoubleValue) getPresentationValue(AM_LOWER_LIMIT);
			if (wv_lowerLimit != null) {
				query = "RESTRICT(" + query + ",GE(" + AM_UPPER_LIMIT + ",SCALEDNUMBER(" + wv_lowerLimit.getDouble() + ")))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$ //$NON-NLS-4$
			}
			IEEEDoubleValue wv_upperLimit = (IEEEDoubleValue) getPresentationValue(AM_UPPER_LIMIT);
			if (wv_upperLimit != null) {
				query = "RESTRICT(" + query + ",LE(" + AM_LOWER_LIMIT + ",SCALEDNUMBER(" + wv_upperLimit.getDouble() + ")))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$ //$NON-NLS-4$
			}
			LogicalValue wv_nonZeroBonusOnly = (LogicalValue) getPresentationValue("NONZEROBONUSONLY"); //$NON-NLS-1$
			if (wv_nonZeroBonusOnly != null && wv_nonZeroBonusOnly.getValue()) {
				query = "RESTRICT(" + query + ",GT(" + AM_BONUS + ",SCALEDNUMBER(0)))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
			}
			setPresentationValue(SalaryGradesOverviewPresentation.GRADELIST, ASPLListFactory.getASPLListValue(getPresentation().getPresentationField(SalaryGradesOverviewPresentation.GRADELIST), dbc.startTransaction(getUserData().getUserID(), true, "", new byte[] {}, true, TransactionMode.READONLY, DDLCapture.OFF, SnapShotIsolationMode.OFF).execQueryAndEndTransaction(query + ",(" + AM_LOWER_LIMIT + ")"), getUserData(), getUserLocale())); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		} catch (ConnectionClosedException e) {
			log(e);
			throw new DBProblem(e);
		} catch (ErrorMessageException e) {
			log(e);
			throw getASPLApplicationException(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doResumeLogic()
	 */
	public void doResumeLogic ( ) throws ASPLException {
		doPresentationLogic();
	}
}