/**
 * 
 */
package be.ASPL.presentationlayer;

import java.util.HashMap;
import java.util.Locale;
import java.util.Properties;

/**
 * @author Erwin
 * 
 */
final class ListRenderer extends Renderer {

	/**
	 * 
	 */
	private static Renderer instance = new ListRenderer();

	/**
	 * @return
	 */
	static Renderer getInstance ( ) {
		return instance;
	}

	/**
	 * Creates the ListRenderer
	 * 
	 */
	private ListRenderer ( ) {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.Renderer#emptyHTML(be.erwinsmout.ASPL.presentationlayer.PresentationField)
	 */
	@Override
	String getEmptyHTML (PresentationField presentationField, Locale locale, Properties presentationLocaleLabels) {
		return ""; //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.Renderer#renderHTML(be.erwinsmout.ASPL.WaarValue, be.erwinsmout.ASPL.presentationlayer.OnLinePresentationFunction, be.erwinsmout.ASPL.presentationlayer.PresentationField)
	 */
	@Override
	String getHTML (PresentationValue presentationFieldValue, OnLinePresentationFunction userFunction, PresentationField presentationField, Locale locale, Properties presentationLocaleLabels) {
		ListValue listValue = (ListValue) presentationFieldValue;
		StringBuilder htmlFieldValue = new StringBuilder(30 * presentationField.getColumnCount() * listValue.getRowList().size());
		int replaceBeginPos, replaceEndPos;
		String fldHtml = ""; //$NON-NLS-1$

		for (HashMap<String, PresentationValue> rowValueMap : listValue.getRowList()) {
			// reset the column iterator
			StringBuilder rowHtml = HTMLPresentationHandler.getLocalizedFormOutputTemplate(presentationField.getRowTemplateText().toString(), presentationLocaleLabels);
			do {
				replaceBeginPos = rowHtml.toString().indexOf("[["); //$NON-NLS-1$
				if (replaceBeginPos >= 0) {
					String rowFieldName = rowHtml.substring(replaceBeginPos + 2);
					replaceEndPos = rowFieldName.indexOf("]]"); //$NON-NLS-1$
					if (replaceEndPos >= 0) {
						rowFieldName = rowFieldName.substring(0, replaceEndPos);
						try {
							fldHtml = Architecture.getArchitectureFieldValue(rowFieldName, userFunction);
						} catch (NotAnArchitectureFieldException e) {
							PresentationField listPresentationFieldColumn;
							try {
								listPresentationFieldColumn = presentationField.getColumn(rowFieldName);
								if (listPresentationFieldColumn != null) {
									PresentationType<?> listPresentationFieldColumnType = listPresentationFieldColumn.getType();
									if (rowValueMap.keySet().contains(rowFieldName)) {
										PresentationValue colValue = rowValueMap.get(rowFieldName);
										fldHtml = listPresentationFieldColumnType.getNakedStringRepresentation(colValue, userFunction, listPresentationFieldColumn, locale);
									} else {
										fldHtml = listPresentationFieldColumnType.getEmptyNakedStringRepresentation(listPresentationFieldColumn, locale);
									}
								} else {
									// not found in listfieldmap, maybe in 'outer' presentation map ...
									HashMap<String, PresentationValue> presentationValues = userFunction.getPresentation().getPresentationValues();
									HashMap<String, PresentationField> presentationFields = userFunction.getPresentation().getPresentationFields();
									if (presentationFields.containsKey(rowFieldName)) {
										listPresentationFieldColumn = presentationFields.get(rowFieldName);
										PresentationType<?> listPresentationFieldColumnType = listPresentationFieldColumn.getType();
										if (presentationValues.keySet().contains(rowFieldName)) {
											PresentationValue colValue = presentationValues.get(rowFieldName);
											fldHtml = listPresentationFieldColumnType.getNakedStringRepresentation(colValue, userFunction, listPresentationFieldColumn, locale);
										} else {
											fldHtml = listPresentationFieldColumnType.getEmptyNakedStringRepresentation(listPresentationFieldColumn, locale);
										}
									} else {
										fldHtml = "##" + rowFieldName + "##"; //$NON-NLS-1$//$NON-NLS-2$
									}
								}
							} catch (NoColumnsInNonListFieldsException e1) {

							}
						}
						rowHtml.delete(replaceBeginPos, replaceBeginPos + replaceEndPos + 4);
						rowHtml.insert(replaceBeginPos, fldHtml);
					}
				}
			} while (replaceBeginPos >= 0);

			do {
				replaceBeginPos = rowHtml.toString().indexOf("{{"); //$NON-NLS-1$
				if (replaceBeginPos >= 0) {
					String rowFieldName = rowHtml.substring(replaceBeginPos + 2);
					replaceEndPos = rowFieldName.indexOf("}}"); //$NON-NLS-1$
					if (replaceEndPos >= 0) {
						rowFieldName = rowFieldName.substring(0, replaceEndPos);
						try {
							fldHtml = Architecture.getArchitectureFieldValue(rowFieldName, userFunction);
						} catch (NotAnArchitectureFieldException e) {
							PresentationField listPresentationFieldColumn;
							try {
								listPresentationFieldColumn = presentationField.getColumn(rowFieldName);
								if (listPresentationFieldColumn != null) {
									if (rowValueMap.keySet().contains(rowFieldName)) {
										PresentationValue colValue = rowValueMap.get(rowFieldName);
										fldHtml = listPresentationFieldColumn.getPresentationRenderer().getHTML(colValue, userFunction, listPresentationFieldColumn, locale, presentationLocaleLabels);
									} else {
										fldHtml = listPresentationFieldColumn.getPresentationRenderer().getEmptyHTML(listPresentationFieldColumn, locale, presentationLocaleLabels);
									}
								} else {
									// not found in listfieldmap, maybe in 'outer' presentation map ...
									HashMap<String, PresentationValue> presentationValues = userFunction.getPresentation().getPresentationValues();
									HashMap<String, PresentationField> presentationFields = userFunction.getPresentation().getPresentationFields();
									if (presentationFields.containsKey(rowFieldName)) {
										listPresentationFieldColumn = presentationFields.get(rowFieldName);
										if (presentationValues.keySet().contains(rowFieldName)) {
											PresentationValue colValue = presentationValues.get(rowFieldName);
											fldHtml = listPresentationFieldColumn.getPresentationRenderer().getHTML(colValue, userFunction, listPresentationFieldColumn, locale, presentationLocaleLabels);
										} else {
											fldHtml = listPresentationFieldColumn.getPresentationRenderer().getEmptyHTML(listPresentationFieldColumn, locale, presentationLocaleLabels);
										}
									} else {
										fldHtml = "##" + rowFieldName + "##"; //$NON-NLS-1$//$NON-NLS-2$
									}
								}
							} catch (NoColumnsInNonListFieldsException e1) {

							}
						}
						rowHtml.delete(replaceBeginPos, replaceBeginPos + replaceEndPos + 4);
						rowHtml.insert(replaceBeginPos, fldHtml);
					}
				}
			} while (replaceBeginPos >= 0);
			htmlFieldValue.append(rowHtml);
		}
		return htmlFieldValue.toString();
	}
}
