package be.ASPL.presentationlayer;

import java.io.IOException;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 * The LogonServlet handles the User logon procedure. The URL to which to return after the logon has completed, is present as the value of the return= parameter in the HttpServletRequest
 * 
 * @version 1.0
 * @author Erwin
 */
public class LogonServlet extends HttpServlet {

	/**
	 * 
	 */
	private static final long serialVersionUID = 4763962818840118976L;

	/**
	 * Processes the logon request. The HttpServletRequest has USERSESSIONID and return parameters
	 * 
	 * @param req
	 *            The request
	 * @param resp
	 *            The response object
	 * @throws ServletException
	 */
	@SuppressWarnings("deprecation")
	private void process (HttpServletRequest req, HttpServletResponse resp) throws ServletException {
		UserSession userSession = Architecture.getUserSession(Long.parseLong(req.getParameter("USERSESSIONID"))); //$NON-NLS-1$
		PresentationFunctionStack functionStack = userSession.getFunctionStack();
		OnLinePresentationFunction userFunction;

		String action = req.getParameter("Action"); //$NON-NLS-1$
		if (action == null) {
			// This means that the logon function has to be initiated
			userFunction = new IdentifyUser();
			userFunction.initialize(req.getContextPath().substring(1), "", null, req.getParameter("return"), "", "IdentifyUser", HTTPArchitectureServlet.getParameterValuesMap(req), new LinkedList<Part>(), this, req.getServletPath()); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
			userFunction.setPresentationMessage(I18N.getString(Architecture.getDefaultLocale(), "LogonServlet.Text")); //$NON-NLS-1$
			functionStack.addLast(userFunction);

			try {
				resp.getWriter().write(userFunction.getHTMLOutput());
			} catch (IOException e1) {
				throw new ServletException(e1.getMessage());
			}
		} else {
			// This means that OK, or cancel, or ... has been pressed on the logon function presentation
			userFunction = (OnLinePresentationFunction) functionStack.getLast();
			try {
				if (action.equalsIgnoreCase("OK")) { //$NON-NLS-1$
					if (userFunction.isHelpPresentationPoppedUp()) {
						userFunction.popoffPresentation();
						userFunction.setHelpPresentationPoppedUp(false);
					} else {
						userFunction.fillInPresentationValuesIgnoringDefaults(HTTPArchitectureServlet.getParameterValuesMap(req), new LinkedList<Part>());
						userFunction.doPresentationLogic();
						userFunction.endPresentationLogic();
						HashMap<String, PresentationValue> presentationValues = userFunction.getPresentation().getPresentationValues();
						PresentationValue userDateFormat_v = presentationValues.get("DATEFORMAT"); //$NON-NLS-1$
						String userDateFormat;
						if (userDateFormat_v == null) {
							userDateFormat = "yyyy'-'MM'-'dd"; //$NON-NLS-1$
						} else {
							userDateFormat = ((StringValue) userDateFormat_v).getString();
						}
						PresentationValue userTimeFormat_v = presentationValues.get("TIMEFORMAT"); //$NON-NLS-1$
						String userTimeFormat;
						if (userTimeFormat_v == null) {
							userTimeFormat = "HH':'mm':'ss"; //$NON-NLS-1$
						} else {
							userTimeFormat = ((StringValue) userTimeFormat_v).getString();
						}

						// logon using server-local userIDs
						UserIDValue userIDValue = ((UserIDValue) presentationValues.get("USERID")); //$NON-NLS-1$
						String userID = userIDValue != null ? userIDValue.getUserID().toLowerCase() : ""; //$NON-NLS-1$
						if (userID.length() > 0) {
							StringValue userPasswordValue = ((StringValue) presentationValues.get("PASSWORD")); //$NON-NLS-1$
							String userPassword = userPasswordValue != null ? userPasswordValue.getString() : ""; //$NON-NLS-1$
							StringValue newPasswordValue = ((StringValue) presentationValues.get("NEWPASSWORD")); //$NON-NLS-1$
							String newPassword = newPasswordValue != null ? newPasswordValue.getString() : ""; //$NON-NLS-1$
							StringValue confirmNewPasswordValue = ((StringValue) presentationValues.get("CONFIRMNEWPASSWORD")); //$NON-NLS-1$
							String confirmNewPassword = confirmNewPasswordValue != null ? confirmNewPasswordValue.getString() : ""; //$NON-NLS-1$

							Architecture.checkPassword(userFunction.getApplicationName(), userID, userPassword, newPassword, confirmNewPassword);
						}

						UserData userData = new UserData(userID, userDateFormat, userTimeFormat, ((StringValue) presentationValues.get("LANGUAGE")), userSession, Architecture.getUserRoles(userFunction.getApplicationName(), userID)); //$NON-NLS-1$
						userSession.setUserData(userData);

						String backto = userFunction.getReturnPath();
						String backparms = userFunction.getQueryString();
						if (backparms == null) {
							backparms = "USERSESSIONID=" + userSession.getSessionID() + "&USERSESSIONTOKEN=" + userSession.getSessionVerificationToken(); //$NON-NLS-1$ //$NON-NLS-2$
						} else {
							backparms = backparms + "&USERSESSIONID=" + userSession.getSessionID() + "&USERSESSIONTOKEN=" + userSession.getSessionVerificationToken(); //$NON-NLS-1$ //$NON-NLS-2$
						}
						functionStack.removeLast();
						String erdurl = resp.encodeRedirectURL(backto + "?" + backparms); //$NON-NLS-1$
						try {
							resp.sendRedirect(erdurl);
						} catch (IOException e2) {
							throw new ServletException(e2.getMessage());
						}
					}
				} else {
					if (action.equalsIgnoreCase("CANCEL")) { //$NON-NLS-1$
						if (userFunction.isHelpPresentationPoppedUp()) {
							userFunction.popoffPresentation();
							userFunction.setHelpPresentationPoppedUp(false);
						} else {
							// go back to menu screen
							Architecture.dropUserSession(userSession.getSessionID());
							String erdurl = resp.encodeRedirectURL(req.getContextPath());
							try {
								resp.sendRedirect(erdurl);
							} catch (IOException e) {
								log(e.getMessage(), e);
								req.getSession().invalidate();
								Architecture.dropUserSession(userSession.getSessionID());
								throw new ServletException(e.getMessage());
							}
						}
					} else {
						if (action.equalsIgnoreCase("HELP")) { //$NON-NLS-1$
							if (!userFunction.isHelpPresentationPoppedUp()) {
								HelpPresentation helpPresentationAnnotation = userFunction.getClass().getAnnotation(HelpPresentation.class);
								if (helpPresentationAnnotation != null) {
									userFunction.popupPresentation(helpPresentationAnnotation.helpPresentationClass());
								} else {
									userFunction.popupPresentation(userFunction.getClass().getSimpleName() + "Help"); //$NON-NLS-1$
								}
								userFunction.setHelpPresentationPoppedUp(true);
							}
						} else {
							userFunction.fillInPresentationValuesIgnoringDefaults(HTTPArchitectureServlet.getParameterValuesMap(req), new LinkedList<Part>());
							userFunction.doActionLogic(action);
							try {
								resp.getWriter().write(userFunction.getHTMLOutput());
							} catch (IOException e1) {
								throw new ServletException(e1.getMessage());
							}
						}
					}
				}
			} catch (ASPLException e3) {
				userFunction.setPresentationMessage(e3);
				try {
					resp.getWriter().write(userFunction.getHTMLOutput());
				} catch (IOException e1) {
					throw new ServletException(e1.getMessage());
				}
			}
		}
	}

	/**
	 * @see javax.servlet.http.HttpServlet#doGet (javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	public void doGet (HttpServletRequest req, HttpServletResponse resp) throws ServletException {
		process(req, resp);
	}

	/**
	 * @see javax.servlet.http.HttpServlet#doPost (javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	public void doPost (HttpServletRequest req, HttpServletResponse resp) throws ServletException {
		process(req, resp);
	}
}
