/**
 * 
 */
package be.ASPL.presentationlayer;

import java.util.Locale;
import java.util.Properties;

/**
 * @author Erwin
 * 
 */
public abstract class Renderer {

	/**
	 * @param presentationField
	 * @return
	 */
	final static String getHTMLControlNameSpec (PresentationField presentationField) {
		return " name=\"" + presentationField.getName() + "\""; //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * @param presentationField
	 * @return
	 */
	final static String getHTMLCssClassSpec (PresentationField presentationField) {
		final String cssClass = presentationField.getCssClass();
		return cssClass.length() > 0 ? (" class=\"" + cssClass + "\"") : ""; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}

	/**
	 * @param presentationField
	 * @param defaultCssClass
	 * @return
	 */
	final static String getHTMLCssClassSpec (PresentationField presentationField, String defaultCssClass) {
		final String cssClass = presentationField.getCssClass();
		final String specifiedOrDefaultCssClass = cssClass.length() > 0 ? cssClass : defaultCssClass;
		return specifiedOrDefaultCssClass.length() > 0 ? (" class=\"" + specifiedOrDefaultCssClass + "\"") : ""; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}

	/**
	 * @param presentationField
	 * @return
	 */
	final static String getHTMLSizeAttributeSpec (PresentationField presentationField) {
		String presentationSizeSpec = presentationField.getPresentationSizeSpec(""); //$NON-NLS-1$
		return presentationSizeSpec.length() > 0 ? " size=\"" + presentationSizeSpec + "\"" : ""; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
	}

	/**
	 * @param presentationField
	 * @return
	 */
	final static String getHTMLTabIndexSpec (PresentationField presentationField) {
		int presentationFieldTabIndex = presentationField.getTabIndex();
		return presentationFieldTabIndex == -1 ? "" : " tabindex=\"" + presentationFieldTabIndex + "\""; //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$
	}

	/**
	 * @param presentationFieldValue
	 * @return
	 */
	final static String processEntityRefs (String presentationFieldValue) {
		StringBuilder stringBuilder = new StringBuilder(presentationFieldValue);
		int p;
		while ((p = stringBuilder.indexOf("\"")) >= 0) { //$NON-NLS-1$
			stringBuilder.deleteCharAt(p).insert(p, "&quot;"); //$NON-NLS-1$
		}
		// while ((p = sb.indexOf("'")) >= 0) { //$NON-NLS-1$
		// sb.deleteCharAt(p).insert(p,"&apost;"); //$NON-NLS-1$
		// }
		while ((p = stringBuilder.indexOf("<")) >= 0) { //$NON-NLS-1$
			stringBuilder.deleteCharAt(p).insert(p, "&lt;"); //$NON-NLS-1$
		}
		while ((p = stringBuilder.indexOf(">")) >= 0) { //$NON-NLS-1$
			stringBuilder.deleteCharAt(p).insert(p, "&gt;"); //$NON-NLS-1$
		}
		// while ((p = sb.indexOf("&")) >= 0) { //$NON-NLS-1$
		// sb.deleteCharAt(p).insert(p,"&amp;"); //$NON-NLS-1$
		// }
		return stringBuilder.toString();
	}

	/**
	 * @param presentationField
	 * @param locale
	 * @param presentationLocaleLabels
	 * @return
	 */
	abstract String getEmptyHTML (PresentationField presentationField, Locale locale, Properties presentationLocaleLabels);

	/**
	 * @param presentationFieldValue
	 * @param userFunction
	 * @param presentationField
	 * @param locale
	 * @param presentationLocaleLabels
	 * @return
	 */
	abstract String getHTML (PresentationValue presentationFieldValue, OnLinePresentationFunction userFunction, PresentationField presentationField, Locale locale, Properties presentationLocaleLabels);

}
