package be.ASPL.presentationlayer;

import java.util.Locale;

import be.SIRAPRISE.util.MyMessageFormat;

/**
 * GenericString is the Presentation layer string type
 * 
 * @author Erwin
 */
public class StringType extends PresentationType<StringValue> {

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#checkValidWaarValue(be.ASPL.presentationlayer.WaarValue, be.ASPL.presentationlayer.UserData)
	 */
	@Override
	public StringValue checkValidWaarValue (PresentationValue value, UserData userData) throws InvalidAttributeValueException {
		if (!(value instanceof StringValue)) {
			Locale locale = userData.getLocale();
			throw new InvalidAttributeValueException(value.toASPLString(userData), this.getClass().getSimpleName(), MyMessageFormat.format(I18N.getString(locale, "DynamicColumnList.InvalidValueClass"), new String[] { getClass().getName(), value.getClass().getName() })); //$NON-NLS-1$
		}
		return (StringValue) value;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#encode(byte[], be.ASPL.presentationlayer.UserData, java.util.Locale)
	 */
	@Override
	public StringValue encode (byte[] partBytes, UserData userData, Locale locale) {
		throw new UnsupportedOperationException();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.waar.AttributeType#validate(java.lang.String)
	 */
	public StringValue encode (String val, UserData userData, Locale locale) {
		return new StringValue(val);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationType#getDefaultPresentationRenderingStyle()
	 */
	@Override
	public Renderer getDefaultPresentationRenderingStyle ( ) {
		return TextBoxRenderer.getInstance();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationAttributeType#decode(be.erwinsmout.ASPL.WaarValue)
	 */
	public String getNakedStringRepresentation (PresentationValue presentationValue, OnLinePresentationFunction userFunction, PresentationField presentationField, Locale locale) {
		StringBuffer stringBuffer = new StringBuffer(((StringValue) presentationValue).getString());
		int p;
		while ((p = stringBuffer.indexOf("\"")) >= 0) { //$NON-NLS-1$
			stringBuffer.deleteCharAt(p).insert(p, "&quot;"); //$NON-NLS-1$
		}
		// while ((p = sb.indexOf("'")) >= 0) { //$NON-NLS-1$
		// sb.deleteCharAt(p).insert(p,"&apost;"); //$NON-NLS-1$
		// }
		while ((p = stringBuffer.indexOf("<")) >= 0) { //$NON-NLS-1$
			stringBuffer.deleteCharAt(p).insert(p, "&lt;"); //$NON-NLS-1$
		}
		while ((p = stringBuffer.indexOf(">")) >= 0) { //$NON-NLS-1$
			stringBuffer.deleteCharAt(p).insert(p, "&gt;"); //$NON-NLS-1$
		}
		// while ((p = sb.indexOf("&")) >= 0) { //$NON-NLS-1$
		// sb.deleteCharAt(p).insert(p,"&amp;"); //$NON-NLS-1$
		// }
		return stringBuffer.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationType#getPresentationRenderingStyle(java.lang.String)
	 */
	@Override
	public Renderer getPresentationRenderingStyle (String renderingStyleName) {
		if (renderingStyleName.equalsIgnoreCase("TEXTAREA")) { //$NON-NLS-1$
			return TextAreaRenderer.getInstance();
		}
		if (renderingStyleName.equalsIgnoreCase("PASSWORD")) { //$NON-NLS-1$
			return PassWordRenderer.getInstance();
		}
		return TextBoxRenderer.getInstance();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#getWaarValue(boolean, java.util.Locale)
	 */
	@Override
	public PresentationValue getWaarValue (boolean b, Locale locale) {
		return new StringValue(I18N.getString(locale, "Logical.True")); //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#getWaarValue(int)
	 */
	@Override
	public PresentationValue getWaarValue (int v, Locale locale) {
		return new StringValue(Integer.toString(v));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#getWaarValue(long, java.util.Locale)
	 */
	@Override
	public PresentationValue getWaarValue (long v, Locale locale) {
		return new StringValue(Long.toString(v));
	}
}
