package be.ASPL.webclient;

import java.util.*;

import be.ASPL.presentationlayer.*;

/**
 * @author Erwin
 */
public class UsersDelete extends UsersFunction {

	/**
	 * Comment for <code>leave</code>
	 */
	private boolean leave = false;

	/**
	 * @throws ASPLException
	 */
	private void processSelectedUser ( ) throws ASPLException {
		PresentationValue wv_userID = getPresentationValue("USERID"); //$NON-NLS-1$
		String userID = ((StringValue) wv_userID).getString().trim();

		UserProperties userProperties = Architecture.getUserProperties(getApplicationName(), userID);

		// switch from 'select relvar' presentation to 'manage some particular relvar's details' presentation
		setPresentation(new UsersDeleteConfirmPresentation());
		setPresentationValue("USERID", wv_userID); //$NON-NLS-1$
		setPresentationValue("DISABLED", new LogicalValue(userProperties.isDisabled())); //$NON-NLS-1$
		setPresentationValue("PWCHANGEREQUIRED", new LogicalValue(userProperties.isPasswordToBeChangedAtNextLogon())); //$NON-NLS-1$
		setPresentationValue("LANGUAGE", userProperties.getLanguage()); //$NON-NLS-1$

		TreeSet<Long> suspiciousLogonAttemptTimes = userProperties.getSuspiciousLogonAttemptTimes();
		ListValue wv_pwFailures = new ListValue("PWFAILURES"); //$NON-NLS-1$
		Iterator<Long> i_suspiciousLogonAttemptTimes = suspiciousLogonAttemptTimes.iterator();
		while (i_suspiciousLogonAttemptTimes.hasNext()) {
			long suspiciousLogonAttemptTime = i_suspiciousLogonAttemptTimes.next().longValue();
			TimeStampValue wv_suspiciousLogonAttemptTime = new TimeStampValue(suspiciousLogonAttemptTime);
			HashMap<String, PresentationValue> m = new HashMap<String, PresentationValue>();
			m.put("PASSWORDFAILURE", wv_suspiciousLogonAttemptTime); //$NON-NLS-1$
			wv_pwFailures.append(m);
		}
		setPresentationValue("PWFAILURES", wv_pwFailures); //$NON-NLS-1$

		saveInFunctionContext("USERID", wv_userID); //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doInitialLogic()
	 */
	public void doInitialLogic ( ) throws ASPLException {
		if (getPresentation().getName().equalsIgnoreCase(UsersSelect) && getPresentationValue("USERID") != null) { //$NON-NLS-1$
			processSelectedUser();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doPresentationLogic()
	 */
	public void doPresentationLogic ( ) throws ASPLException {
		switch (getPresentation().getName().toUpperCase()) {
			case "USERSSELECT": { //$NON-NLS-1$
				// selection criteria entered, inquire catalog and switch to managedetails presentation
				processSelectedUser();
				break;
			}
			case "USERSDELETE": { //$NON-NLS-1$
				Architecture.removeWaarUser(getApplicationName(), ((StringValue) (PresentationValue) getFromFunctionContextMandatory("USERID")).getString().trim()); //$NON-NLS-1$
				leave = true;
				// Modifications to screen currently not supported
				break;
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#getInitialPresentationClass()
	 */
	public Class<? extends Presentation> getInitialPresentationClass ( ) {
		if (getUserRoles().contains(Architecture.ASPLADMINISTRATOR_ROLENAME)) {
			return UsersSelectPresentation.class;
		} else {
			return UsersVoidPresentation.class;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#getPresentationFunctionDescription()
	 */
	public String getPresentationFunctionDescription () {
		return "Delete a WAAR User ID"; //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#leaveFunctionOnExit()
	 */
	public boolean leaveFunctionOnExit ( ) {
		return leave;
	}
}
