/**
 * 
 */
package be.SIRAPRISE.birdwatcher;

import be.ASPL.presentationlayer.StringValue;
import be.ASPL.presentationlayer.LogicalValue;
import be.ASPL.presentationlayer.ASPLApplicationException;
import be.SIRAPRISE.client.ConnectionClosedException;
import be.SIRAPRISE.client.DBConnection;
import be.SIRAPRISE.client.RelationValueSelectorFactory;
import be.SIRAPRISE.client.TransactionMode;
import be.SIRAPRISE.messages.ErrorMessageException;
import be.SIRAPRISE.typeimplementations.Relation;
import be.SIRAPRISE.typeimplementations.TYPENAMES;
import be.SIRAPRISE.util.BracketParser;
import be.SIRAPRISE.util.MyString;

/**
 * @author Erwin
 * 
 */
public abstract class BWSpeciesFunction extends BWWebClientFunction {

	/**
	 * @return
	 * @throws ASPLApplicationException
	 */
	Relation getMatchingSpecies ( ) throws ASPLApplicationException {

		String speciesQuery;
		LogicalValue wv_possibleSpeciesOnly = (LogicalValue) getPresentationValue(ExistingSpeciesOverviewPresentation.POSSIBLESPECIESONLY);
		if (wv_possibleSpeciesOnly != null && wv_possibleSpeciesOnly.getValue()) {
			speciesQuery = RELVARNAMES.POSSIBLESPECIES;
		} else {
			if (getPresentationValue(ATTRIBUTENAMES.SPECIESNAME) == null && getPresentationValue(ATTRIBUTENAMES.NLSPECIESNAME) == null && getPresentationValue(ExistingSpeciesOverviewPresentation.SPECIESNAMEPATTERN) == null && getPresentationValue(ExistingSpeciesOverviewPresentation.NLSPECIESNAMEPATTERN) == null) {
				throw new PleaseEnterSpeciesCriteriaException();
			}
			speciesQuery = RELVARNAMES.EXISTINGSPECIES;
		}

		StringValue name = (StringValue) getPresentationValue(ATTRIBUTENAMES.SPECIESNAME);
		StringValue namePattern = (StringValue) getPresentationValue(ExistingSpeciesOverviewPresentation.SPECIESNAMEPATTERN);
		if (name != null) {
			speciesQuery = "JOIN(" + speciesQuery + "," + RelationValueSelectorFactory.getUnaryRelationValueSelectorExpressionText(MyString.getNames(name.getString(), ','), ATTRIBUTENAMES.SPECIESNAME, TYPENAMES.STRING) + ")"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		} else {
			if (namePattern != null) {
				speciesQuery = "RESTRICT(" + speciesQuery + "," + "MATCHES(" + ATTRIBUTENAMES.SPECIESNAME + ",STRING(" + BracketParser.meta(namePattern.getString()) + ")))"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
			}
		}

		StringValue nlName = (StringValue) getPresentationValue(ATTRIBUTENAMES.NLSPECIESNAME);
		StringValue nlNamePattern = (StringValue) getPresentationValue(ExistingSpeciesOverviewPresentation.NLSPECIESNAMEPATTERN);
		if (nlName != null) {
			speciesQuery = "SEMIJOIN(" + speciesQuery + ",JOIN(" + RELVARNAMES.NLSPECIESNAME + "," + RelationValueSelectorFactory.getUnaryRelationValueSelectorExpressionText(MyString.getNames(nlName.getString(), ','), ATTRIBUTENAMES.NLSPECIESNAME, TYPENAMES.STRING) + "))"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			speciesQuery = "GROUP(JOIN(" + speciesQuery + "," + RELVARNAMES.NLSPECIESNAME + "),NLSPECIESNAMELIST(" + ATTRIBUTENAMES.LANGUAGE + "," + ATTRIBUTENAMES.NLSPECIESNAME + "))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$//$NON-NLS-5$
		} else {
			if (nlNamePattern != null) {
				String nlNamePatternCondition = "MATCHES(" + ATTRIBUTENAMES.NLSPECIESNAME + ",STRING(" + BracketParser.meta(nlNamePattern.getString()) + "))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
				String restrictCondition;
				LogicalValue wv_onlyInLanguage = (LogicalValue) getPresentationValue("ONLYINLANGUAGE"); //$NON-NLS-1$
				if (wv_onlyInLanguage != null && wv_onlyInLanguage.getValue()) {
					restrictCondition = "AND(EQ(THE_ISO2(" + ATTRIBUTENAMES.LANGUAGE + "),NAME(" + BracketParser.meta(((StringValue) getPresentationValue("LANGUAGE")).getString()) + "))," + nlNamePatternCondition + ")"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
				} else {
					restrictCondition = nlNamePatternCondition;
				}
				speciesQuery = "SEMIJOIN(" + speciesQuery + ",RESTRICT(" + RELVARNAMES.NLSPECIESNAME + "," + restrictCondition + "))"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$//$NON-NLS-4$
				speciesQuery = "GROUP(JOIN(" + speciesQuery + "," + RELVARNAMES.NLSPECIESNAME + "),NLSPECIESNAMELIST(" + ATTRIBUTENAMES.LANGUAGE + "," + ATTRIBUTENAMES.NLSPECIESNAME + "))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$//$NON-NLS-5$
			} else {
				speciesQuery = "LEFTJOIN(" + speciesQuery + ",GROUP(" + RELVARNAMES.NLSPECIESNAME + ",NLSPECIESNAMELIST(" + ATTRIBUTENAMES.LANGUAGE + "," + ATTRIBUTENAMES.NLSPECIESNAME + ")),NLSPECIESNAMELIST(RELATION(HEADING(" + ATTRIBUTENAMES.LANGUAGE + "(LANGUAGE)" + ATTRIBUTENAMES.NLSPECIESNAME + "(STRING))BODY())))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$//$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$
			}
		}

		speciesQuery = "EXTEND(" + speciesQuery + ",ISPOSSIBLE(EQ(TABLE_DEE,PROJECT(SEMIJOIN(RENAME(" + RELVARNAMES.POSSIBLESPECIES + ",(SPECIESNAME,POSSIBLESPECIESNAME)),RELATION(HEADING(POSSIBLESPECIESNAME(STRING))BODY(TUPLE(POSSIBLESPECIESNAME(" + ATTRIBUTENAMES.SPECIESNAME + "))))),()))))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
		
		try (DBConnection dbc = getDBConnection()) {
			return dbc.startTransaction(getUserData().getUserID(), true, "", new byte[] {}, TransactionMode.READONLY).execQueryAndEndTransaction(speciesQuery); //$NON-NLS-1$
		} catch (ConnectionClosedException e) {
			log(e);
			throw new ASPLApplicationException(e);
		} catch (ErrorMessageException e) {
			log(e);
			throw new ASPLApplicationException(e);
		}
	}
}
