/**
 * 
 */
package be.SIRAPRISE.birdwatcher;

import java.io.IOException;
import java.util.Set;
import java.util.logging.*;

import be.ASPL.presentationlayer.*;
import be.SIRAPRISE.client.*;
import be.SIRAPRISE.messages.CommunicationProtocolException;
import be.SIRAPRISE.util.Localizer;
import be.SIRAPRISE.util.MyString;

/**
 * @author Erwin
 * 
 */
public abstract class BWWebClientFunction extends OnLinePresentationFunction {

	/**
	 * 
	 */
	private static Set<String> clientSigningAlgorithmNames;

	/**
	 * 
	 */
	private static Set<String> cryptoAlgorithmNames;

	/**
	 * 
	 */
	private static SiraPriseServer server;

	/**
	 * 
	 */
	static final String CLIENTNAME = "BIRDWATCHER"; //$NON-NLS-1$

	/**
	 * @return
	 */
	private synchronized static Set<String> getClientSigningAlgorithmNames ( ) {
		return (clientSigningAlgorithmNames == null ? (clientSigningAlgorithmNames = getClientSigningAlgorithmNamesFromConfig()) : clientSigningAlgorithmNames);
	}

	/**
	 * @return
	 */
	private static Set<String> getClientSigningAlgorithmNamesFromConfig ( ) {
		return MyString.getNames(DBConnectionProperties.getInstance().getProperty(DBConnectionProperties.SIGNINGALGORITHMS, DBConnectionProperties.DEFAULTSIGNATUREALGORITHMNAMESDEFAULT), ',');
	}

	/**
	 * @return
	 */
	private synchronized static Set<String> getCryptoAlgorithmNames ( ) {
		return (cryptoAlgorithmNames == null ? (cryptoAlgorithmNames = getcryptoAlgorithmNamesFromConfig()) : cryptoAlgorithmNames);
	}

	/**
	 * @return
	 */
	private static Set<String> getcryptoAlgorithmNamesFromConfig ( ) {
		return MyString.getNames(DBConnectionProperties.getInstance().getProperty(DBConnectionProperties.CRYPTOALGORITHMS, DBConnectionProperties.DEFAULTENCRYPTIONALGORITHMNAMESDEFAULT), ',');
	}

	/**
	 * @return
	 */
	private synchronized static SiraPriseServer getServer ( ) {
		return (server == null ? (server = new SiraPriseServer(DBConnectionProperties.getInstance().getProperty(DBConnectionProperties.HOST, DBConnectionProperties.HOSTDEFAULT), Integer.parseInt(DBConnectionProperties.getInstance().getProperty(DBConnectionProperties.PORT, DBConnectionProperties.PORTDEFAULT)), Integer.parseInt(DBConnectionProperties.getInstance().getProperty(DBConnectionProperties.MONITORPORT, DBConnectionProperties.MONITORPORTDEFAULT)))) : server);
	}

	/**
	 * @return
	 * @throws ASPLApplicationException
	 */
	final DBConnection getDBConnection ( ) throws ASPLApplicationException {
		try {
			return new OneShotDBConnection(getServer(), getClientSigningAlgorithmNames(), CLIENTNAME, getCryptoAlgorithmNames(), BirdwatcherSigner.getInstance());
		} catch (IOException | CommunicationProtocolException | DBException e) {
			log(e);
			throw new ASPLApplicationException(e);
		}
	}

	/**
	 * 
	 */
	final void setDefaultLanguageFromUserContext ( ) {
		try {
			setPresentationValue(ATTRIBUTENAMES.LANGUAGE, new StringValue(getUserLocale().getLanguage()));
		} catch (ASPLException e) {
			Logger.getLogger(this.getClass().getName()).log(Level.INFO, "", e); //$NON-NLS-1$
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.OnLinePresentationFunction#getApplicationLocalizer()
	 */
	@Override
	protected final Localizer getApplicationLocalizer ( ) {
		return I18N.getInstance();
	}
}
