package be.SIRAPRISE.birdwatcher;

import java.util.Locale;

import be.ASPL.presentationlayer.ASPLListFactory;
import be.ASPL.presentationlayer.DateValue;
import be.ASPL.presentationlayer.ASPLException;
import be.ASPL.presentationlayer.InitialPresentation;
import be.ASPL.presentationlayer.OnLinePresentationFunctionCharacteristics;
import be.ASPL.presentationlayer.PresentationValue;
import be.SIRAPRISE.typeimplementations.Relation;
import be.SIRAPRISE.util.ISODateFormat;

/**
 * The function to produce an overview of defined attributes
 * 
 * @author Erwin
 */
@InitialPresentation(initialPresentationClass = WalksOverviewPresentation.class)
@OnLinePresentationFunctionCharacteristics(leaveFunctionOnExit = false, descriptionLocalizationKey = "WalksManage.Text")
public class WalksManage extends BWWalksFunction {

	/**
	 * @throws ASPLException
	 */
	private void processSelectionPresentation ( ) throws ASPLException {
		Relation r = getMatchingWalks();
		if (r.size() == 1) {
			DateValue wv_walkDate = new DateValue(ISODateFormat.parseISO(r.iterator().next().valueText(ATTRIBUTENAMES.WALKDATE_I)));
			PresentationValue wv_language = getPresentationValue(ATTRIBUTENAMES.LANGUAGE);
			setPresentation(new WalksManageDetailsPresentation());
			setPresentationValue(ATTRIBUTENAMES.WALKDATE, wv_walkDate);
			setPresentationValue(ATTRIBUTENAMES.LANGUAGE, wv_language);
			refreshInfo();
		} else {
			Locale locale = getUserLocale();
			setPresentationValue("WALKSLIST", ASPLListFactory.getASPLListValue(getPresentation().getPresentationField("WALKSLIST"), r, getUserData(), locale)); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}

	/**
	 * @throws ASPLException
	 * 
	 */
	private void refreshInfo ( ) throws ASPLException {
		Locale locale = getUserLocale();
		setPresentationValue("SPOTSLIST", ASPLListFactory.getASPLListValue(getPresentation().getPresentationField("SPOTSLIST"), getMatchingSpots(), getUserData(), locale)); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doInitialLogic()
	 */
	public void doInitialLogic ( ) throws ASPLException {
		setDefaultLanguageFromUserContext();

		if (getPresentationValue(ATTRIBUTENAMES.WALKDATE) != null || getPresentationValue("FROMDATE") != null || getPresentationValue("TODATE") != null || getPresentationValue(ATTRIBUTENAMES.SPECIESNAME) != null) { //$NON-NLS-1$ //$NON-NLS-2$
			processSelectionPresentation();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doPresentationLogic()
	 */
	public void doPresentationLogic ( ) throws ASPLException {
		if (getPresentation().getName().equalsIgnoreCase("WalksOverview")) { //$NON-NLS-1$
			processSelectionPresentation();
		} else {
			refreshInfo();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doResumeLogic()
	 */
	public void doResumeLogic ( ) throws ASPLException {
		doPresentationLogic();
	}
}